﻿using System;
using System.Collections.Generic;

namespace MyUni
{
    class Exam
    {
        private string name, uid;
        private ushort mark;
        private bool isCompitino;

        public Exam(string name, ushort mark, bool isCompitino, string uid = null)
        {
            this.name = name;
            this.mark = mark;
            this.isCompitino = isCompitino;
            if (uid == null)
            {
                uid = Basic.generateRandomID();
            }
            this.uid = uid;
        }

        public override bool Equals(object obj)
        {
            Exam exam = obj as Exam;
            return exam != null &&
                   name.ToLower() == exam.name.ToLower();
        }

        public string getName()
        {
            return name;
        }

        public ushort getMark()
        {
            return mark;
        }

        public bool getIsCompitino()
        {
            return isCompitino;
        }

        public string getUID()
        {
            return uid;
        }

        public void saveToDisk(string path, string file)
        {
            Basic.writeToFile(path, file, new string[] { "uid=" + getUID(), "name=" + getName(), "mark=" + getMark(), "isCompitino=" + getIsCompitino() });
        }
    }

    class University
    {
        private string name, uid;
        private List<Exam> exams = new List<Exam>();

        public University(string name, string uid = null)
        {
            this.name = name;
            if (uid == null)
            {
                uid = Basic.generateRandomID();
            }
            this.uid = uid;
        }

        public string getUID()
        {
            return uid;
        }

        public override bool Equals(object obj)
        {
            University university = obj as University;
            return university != null &&
                   name.ToLower() == university.name.ToLower();
        }

        public string getName()
        {
            return name;
        }

        public bool addExam(Exam exam)
        {
            foreach (Exam current in exams)
            {
                if (current.Equals(exam)) return false;
            }

            exams.Add(exam);
            return true;
        }

        public void printExams()
        {
            foreach (Exam current in exams)
            {
                Console.WriteLine((current.getIsCompitino() == true ? "Compitino" : "Exam") + " of " + current.getName() + " with mark " + current.getMark());
            }
        }

        public float getExamsAvg()
        {
            uint sum = 0;
            uint i = 0;
            foreach (Exam current in exams)
            {
                if (!current.getIsCompitino())
                {
                    sum += current.getMark();
                    i++;
                }
            }

            return sum / i;
        }

        public void saveToDisk(string path, string file)
        {
            Basic.writeToFile(path, file, new string[] { "uid=" + getUID(), "name=" + getName() });
        }
    }

    class Student
    {
        private string name, surname, uid;
        private List<University> university = new List<University>();

        public Student(string name, string surname, string uid = null)
        {
            this.name = name;
            this.surname = surname;
            if(uid == null)
            {
                uid = Basic.generateRandomID();
            }
            this.uid = uid;
        }

        public string getName()
        {
            return name;
        }

        public string getSurname()
        {
            return surname;
        }

        public string getUID()
        {
            return uid;
        }

        public bool addUniversity(string name)
        {
            University add = new University(name);
            foreach (University current in university)
            {
                if (current.Equals(add)) return false;
            }

            university.Add(add);
            return true;
        }

        public bool addExam(string name, ushort mark, bool isCompitino, string university)
        {
            foreach (University current in this.university)
            {
                if (current.getName().ToLower().Equals(university.ToLower()))
                {
                    return current.addExam(new Exam(name, mark, isCompitino));
                }
            }

            return false;
        }

        public University searchUniversity(string name)
        {
            foreach (University current in university)
            {
                if (current.getName().ToLower().Equals(name.ToLower())) return current;
            }

            return null;
        }

        public List<University> GetUniversities()
        {
            return university;
        }

        public void printUniversities()
        {
            if (university.Count > 0)
            {
                ushort i = 0;
                foreach (University current in university)
                {
                    Console.WriteLine(i + ": " + current.getName());
                    i++;
                }
            }
            else
            {
                Console.WriteLine("No university for this student.");
            }
        }

        public void printExams(string university)
        {
            if (this.university.Count > 0)
            {
                foreach (University current in this.university)
                {
                    if (current.getName().ToLower().Equals(university.ToLower()))
                    {
                        current.printExams();
                        break;
                    }
                }
            }
            else
            {
                Console.WriteLine("No university for this student.");
            }
        }

        public float getExamsAvg(string university)
        {
            if (this.university.Count > 0)
            {
                foreach (University current in this.university)
                {
                    if (current.getName().ToLower().Equals(university.ToLower()))
                    {
                        return current.getExamsAvg();
                    }
                }
            }

            return 0.00f;
        }

        public void saveToDisk(string path, string file)
        {
            Basic.writeToFile(path, file, new string[] { "uid=" + getUID(), "name=" + getName(), "surname=" + getSurname() });
        }
    }

    class Basic
    {
        public static string inputValue(string message)
        {
            Console.Write(message);
            return Console.ReadLine();
        }

        public static void writeToFile(string path, string file, string[] content)
        {
            Basic.createFolder(path);
            System.IO.File.WriteAllLines(@path + @"\" + file, content);
        }

        public static string[] readFromFile(string path)
        {
            return System.IO.File.ReadAllLines(@path);
        }

        public static void createFolder(string path)
        {
            System.IO.Directory.CreateDirectory(path);
        }

        public static bool existsFolder(string path)
        {
            return System.IO.Directory.Exists(path);
        }

        public static string generateRandomID()
        {
            return DateTime.UtcNow.Year.ToString() +
                        DateTime.UtcNow.Month.ToString() +
                        DateTime.UtcNow.Day.ToString() +
                        DateTime.UtcNow.Hour.ToString() +
                        DateTime.UtcNow.Minute.ToString() +
                        DateTime.UtcNow.Second.ToString() +
                        DateTime.UtcNow.Millisecond.ToString();
        }
    }

    class Program
    {
        static List<Student> students = new List<Student>();
        static Student current = null;

        static void Main(string[] args) {
            Console.WriteLine("MyUni C# edition developed by Francesco Sorge\n");

            printMenu();
        }

        static void printMenu()
        {
            bool exit = false;

            do
            {
                if (current == null)
                {
                    chooseStudent();
                }
                else
                {
                    Console.WriteLine();
                    Console.WriteLine("Student selected: " + current.getName() + " " + current.getSurname());

                    Console.WriteLine("Main menu");
                    string[] options = { "Add university", "Print universities", "Add exam mark", "Print exams", "Get average", "Change student", "Exit" };

                    ushort i = 1;
                    foreach(string current in options)
                    {
                        Console.WriteLine("\t" + i + ". " + current);
                        i++;
                    }

                    bool correctValue = false;
                    ushort option = 0;

                    do
                    {
                        try
                        {
                            option = ushort.Parse(Basic.inputValue("Choose a number from range [1-" + options.Length + "]: "));
                            correctValue = true;
                        }
                        catch (Exception)
                        {
                            Console.WriteLine("Invalid number input. Try again.");
                        }
                    } while (!correctValue);

                    switch(option)
                    {
                        case 1:
                            addUniversity();
                            break;
                        case 2:
                            current.printUniversities();
                            break;
                        case 3:
                            addExam();
                            break;
                        case 4:
                            current.printExams(Basic.inputValue("University name: "));
                            break;
                        case 5:
                            Console.WriteLine("Average is " + current.getExamsAvg(Basic.inputValue("University name: ")));
                            break;
                        case 6:
                            chooseStudent();
                            break;
                        case 7:
                            exit = true;
                            break;
                    }
                }
            } while (!exit);

            Console.WriteLine("Exiting MyUni");
        }

        static void addExam()
        {
            Console.WriteLine("Adding new exam");
            String uniName = Basic.inputValue("University name: ");
            if (current.addExam(Basic.inputValue("Name: "), ushort.Parse(Basic.inputValue("Mark: ")), Basic.inputValue("Compitino [y/n]: ") == "y" ? true : false, uniName))
            {
                //Exam created;
                //created.saveToDisk(@"users\" + current.getUID() + @"\university\" + created.getUID(), "details.txt");
                Console.WriteLine("Exam added correctly");
            }
            else
            {
                Console.WriteLine("Exam not added because university does not exists or exam already in your exams list.");
            }
        }

        static void addUniversity()
        {
            if (current.addUniversity(Basic.inputValue("University name: ")))
            {
                University created = current.GetUniversities()[current.GetUniversities().Count-1];
                created.saveToDisk(@"users\" + current.getUID() + @"\university\" + created.getUID(), "details.txt");
                Console.WriteLine("University added correctly");
            }
            else
            {
                Console.WriteLine("Error, university already exists.");
            }
        }

        static void chooseStudent()
        {
            if (students.Count > 0)
            {
                Console.WriteLine("Available students:");
                ushort i = 1;
                foreach (Student current in students)
                {
                    Console.WriteLine("\t" + i + ". " + current.getName() + " " + current.getSurname());
                    i++;
                }
                Console.WriteLine("\t" + i + ". Add new student");

                bool correctValue = false;
                ushort selected = 0;

                do
                {
                    try
                    {
                        selected = ushort.Parse(Basic.inputValue("Select a student in range [1-" + i + "]: "));
                        correctValue = true;
                    }
                    catch (Exception)
                    {
                        Console.WriteLine("Invalid number input. Try again.");
                    }
                } while (!correctValue);

                if (selected == i)
                {
                    newStudent();
                }
                else
                {
                    current = students[selected-1];
                    Console.WriteLine("Selected " + current.getName() + " " + current.getSurname());
                }
            }
            else
            {
                Console.WriteLine("No student available. Creating a new one.");
                newStudent();
            }
        }

        static void newStudent()
        {
            Console.WriteLine("Enter new user details.");
            string name = Basic.inputValue("Name: ");
            string surname = Basic.inputValue("Surname: ");

            Student newStudent = new Student(name, surname);
            students.Add(newStudent);

            newStudent.saveToDisk(@"users\" + newStudent.getUID(), @"details.txt");
            current = newStudent;
        }
    }
}
